; $Id: dm_check_macskidney.pro,v 1.9 2016/04/07 17:45:21 ymqiu Exp $
;#######################################################################
; 
; NAME:
;  dm_check_macskidney
;
; PURPOSE:
;  check macs kideney angle to see if it is out of range
;  
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-6102
;  United States
;  yiming.qiu@nist.gov
;  April, 2016
; 
; HISTORY:
; 02/2013: use NG0 limits
; 04/2013: update with BT9 limits
; 08/2013: update after installing the proximity sensor stopper
; 04/2016: add 17 meV
; 
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

;check kidney position, return the index of good kidney angles
;parameters:
;  kidney:      kidney angles [nkid], software angle
;  Ei:          Ei values [nkid]  
;  count:       return the number of good kidney angles
;keyword:
;  complement:  return the index of bad kidney angles
;  ncomplement: return the number of bad kidney angles
;  limits:      save the limits [ne,3], hardware angle
;  llim:        [nkid], lower kidney range for Ei, software angle
;  ulim:        [nkid], upper kidney range for Ei, software angle
;  offset:      hardware offset angle, it corresponds to the hardware angle when software angle is 0
;  extra keywords are for interpol routine
function dm_check_macskidney,kidney,Ei,count,complement=complement,ncomplement=ncomplement,limits=limits,llim=llim,ulim=ulim,offset=offset,_ref_extra=_extra
    if n_elements(offset) eq 0 then offset=1.44  ;as measured on 1/3/2014
    limits = [[2.35000,      31.0227,      46.1857],$
              [2.50000,      23.7756,      42.7142],$
              [2.75000,      12.3107,      37.5954],$
              [3.00000,      3.72030,      33.0017],$
              [3.25000,     -3.13990,      29.0768],$
              [3.50000,     -8.68440,      25.2239],$
              [3.75000,     -13.2946,      21.6473],$
              [4.00000,     -17.0813,      18.1316],$
              [4.25000,     -20.2266,      15.2626],$
              [4.50000,     -22.8800,      12.4599],$
              [4.75000,     -25.3842,      9.64620],$
              [5.00000,     -27.4793,      7.31340],$
              [5.25000,     -29.3809,      4.67660],$
              [5.50000,     -31.1498,      2.03430],$
              [5.75000,     -32.8745,    -0.265300],$
              [6.00000,     -34.1846,     -2.38250],$
              [6.25000,     -35.5445,     -4.43340],$
              [6.50000,     -36.7496,     -6.35160],$
              [6.75000,     -37.8275,     -8.13710],$
              [7.00000,     -38.7396,     -9.92260],$
              [7.25000,     -39.6130,     -11.0669],$
              [7.50000,     -40.3261,     -12.7861],$
              [7.75000,     -40.9674,     -14.2344],$
              [8.00000,     -41.8519,     -15.6661],$
              [8.25000,     -42.5705,     -17.0591],$
              [8.50000,     -44.6656,     -18.3582],$
              [8.75000,     -44.5163,     -19.6352],$
              [9.00000,     -44.4113,     -20.8679],$
              [9.25000,     -44.7098,     -22.0177],$
              [9.50000,     -45.5279,     -23.1012],$
              [9.75000,     -45.9259,     -24.2399],$
              [10.0000,     -46.1857,     -25.3400],$
              [10.2500,     -46.6833,     -26.3460],$
              [10.5000,     -47.0647,     -27.3853],$
              [10.7500,     -47.4295,     -28.1371],$
              [11.0000,     -47.7059,     -28.9994],$
              [11.2500,     -48.0542,     -30.0332],$
              [11.5000,     -48.3195,     -30.8789],$
              [11.7500,     -48.6457,     -31.7634],$
              [12.0000,     -49.0658,     -32.6036],$
              [12.2500,     -49.1763,     -33.4052],$
              [12.5000,     -49.4085,     -33.9635],$
              [12.7500,     -49.6241,     -34.8093],$
              [13.0000,     -49.8342,     -35.5445],$
              [13.2500,     -50.0829,     -36.0641],$
              [13.5000,     -50.2875,     -36.8104],$
              [13.7500,     -50.5473,     -37.4295],$
              [14.0000,     -50.7850,     -38.0873],$
              [14.2500,     -50.9674,     -38.7175],$
              [14.5000,     -51.2106,     -39.3311],$
              [14.7500,     -51.4041,     -40.0111],$
              [15.0000,     -51.5865,     -40.5915],$
              [15.2500,     -51.7468,     -41.1609],$
              [15.5000,     -51.8574,     -41.9292],$
              [15.7500,     -51.9127,     -42.2056],$
              [16.0000,     -52.0509,     -42.6976]];,$
              ;[17.0000,     -54-1.5+offset,-48.2+1.5+offset]]
    limits = transpose(limits)
    n_kid  = n_elements(kidney)
    count  = n_kid & ncomplement = 0 & complement = -1
    if n_kid eq 0 then return,-1
    if n_elements(Ei) ne n_kid then return,-1
    llim   = interpol(limits[*,1],limits[*,0],Ei,_extra=_extra)-offset
    ulim   = interpol(limits[*,2],limits[*,0],Ei,_extra=_extra)-offset
    index  = where((kidney ge llim) and (kidney le ulim),count,complement=complement,ncomplement=ncomplement)
    return,index
end